
#ifndef _P1_IMAGESDK_CAPTURE_IMAGE_
#define _P1_IMAGESDK_CAPTURE_IMAGE_

#include <C_P1ImageCommonTypes.h>
#include <C_P1ImageCommonLinkage.h>

#include "P1ImageRect.hpp"
#include "P1ImageConvertConfig.hpp"
#include "P1ImageDecodeConfig.hpp"
#include "P1ImageBitmapImage.hpp"
#include "P1ImageImageTag.hpp"
#include "P1ImageGpsImu.hpp"

#include <string>

namespace P1
{
namespace ImageSdk
{
	class RawImagePimpl; // forward decl

	/**
     * A handle to an IIQ or DNG image file, either on the file system or in a
     * memory buffer
     */
	class EXPORT_API RawImage
	{
	public:

		// Constructors & assignments

		/** @brief Constructs an empty and invalid RawImage object */
		RawImage();

		/** @brief Construct a RawImage from a file path */
		RawImage(std::string iiqDngPath);

#ifdef _MSC_VER
		/** @brief Construct a RawImage from a file path using UTF-16 based wchar_t - Windows only! */
		RawImage(std::wstring iiqDngPath);
#endif

		/** @brief Construct a RawImage from a memory pointer, to a complete in-memory IIQ / DNG file */
		RawImage(std::shared_ptr<const uint8_t> buffer, uint32_t bufferByteSize);

		RawImage(const RawImage& other);
		RawImage& operator= (const RawImage& other);

		// Acccessors

		/** @brief Get the image width, in pixels, of the exposed sensor area */
		uint32_t Width() const;

		/** @brief Get the image height, in pixels, of the exposed sensor area */
		uint32_t Height() const;

		/** @brief Get full width of the raw sensor output in pixels (including un-exposed area) */
		uint32_t RawSensorWidth() const;

		/** @brief Get the full height of the raw sensor output in pixels (including un-exposed area) */
		uint32_t RawSensorHeight() const;

		/** @brief Get the image dimensions, in pixels, of the exposed sensor area */
		Size GetDimensions() const;

		// Meta data

		/** @brief Extract and return meta data from the image container file (IIQ / DNG) */
		ImageTag GetTag(TagId id) const;

		/** @brief Check if a certain meta data tag exists in the image file */
		bool TagExists(TagId id) const;

		/**
		 * @brief Extract any RGB bitmap preview / thumbnail from meta data
		 * 
		 * IIQ and DNG files might contain a preview of the full image. This method
		 * extracts that image.
		 * 
		 * @param alloc An *optional* memory allocator. Default is standard C++ `new`. 
		 */
		BitmapImage GetPreview(std::allocator<uint8_t> alloc = std::allocator<uint8_t>());


		// Processing

		/**
		 * @brief Extract the Bayer data from the IIQ / DNG container file.
		 * 
		 * This operation consist of a a decompression stage and a calibration stage.
		 * 
		 * @param config The configration of the static calibration options to be applied the sensor output
		 * @param alloc An *optional* memory allocator. Default is standard C++ `new`. 
		 */
		SensorBayerOutput Decode(const DecodeConfig& config, std::allocator<uint8_t> alloc=std::allocator<uint8_t>()) const;

		/**
		 * @brief Extract and de-mosaic (RAW convert) the Bayer data into an RGB bitmap image.
		 * 
		 * Use the @ref ConvertConfig to control cropping and scaling.
		 * 
		 * @param config A configuration object specifying optional crop and scaling
		 * @param alloc An *optional* memory allocator. Default is standard C++ `new`. 
		 */
		BitmapImage Convert(const ConvertConfig& config, std::allocator<uint8_t> alloc=std::allocator<uint8_t>()) const;


		/**
		* @brief Convenience method to get the XMP meta data
		* 
		* This method equivalent to using GetTag() to get the XmpPacket tag, and trimming the output
		* for extranous white space.
		* 
		* @param alloc An *optional* memory allocator. Default is standard C++ `new`. 
		*/
		std::string GetXmpPacket(std::allocator<char> alloc = std::allocator<char>()) const;

		//bool HasGpsData() const;

		/**
		* @brief Extract the GPS data embedded in the IIQ files XMP meta data section, if any
		* 
		* This is a convenience method. It is equal to using the GetXmpPacket method
		* and paring the GPS data tags.
		* 
		* If no GPS data is embedded in the IIQ file, this method will throw!
		*/
		GpsData GetGpsData() const;

		
		//bool HasImuData() const;
		
		/**
		* @brief Extract the IMU data embedded in the IIQ files XMP meta data section, if any
		* 
		* This is a convenience method. It is equal to using the GetXmpPacket method
		* and paring the IMU data tags.
		* 
		* If no IMU data is embedded in the IIQ file, this method will throw!
		*/
		ImuData GetImuData() const;

	private:

		std::shared_ptr<RawImagePimpl> mPimpl;
	};

} // namespace ImageSdk
} // namespace P1

#endif // _P1_IMAGESDK_CAPTURE_IMAGE_
